// specification.h

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "PL0.h"


/*  PL/0语言的语法定义
Program		::= Block .
Block		::= ConstDecl VarDecl ProcDecSeq Stmt
ConstDecl	::= const ConstDefSeq ; | epsilon
ConstDefSeq	::= ConstDef | ConstDefSeq , ConstDef
ConstDef	::= ident = number
VarDecl		::= var VarDecSeq ; | epsilon
VarDecSeq 	::= ident | VarDecSeq , VarDecSeq			//此处文法与原文法略有不同，但定义同一个语言。
ProcDecSeq	::= ProcDecSeq ProcDecl | epsilon
ProcDecl 	::= procedure ident ; Block ;
Stmt		::= ident := Exp | call ident | begin StmtSeq end | 
		    if Cond then Stmt | while Cond do Stmt | epsilon
StmtSeq		::= Stmt | StmtSeq ; Stmt
Cond		::= odd Exp | Exp RelOp Exp
RelOp		::= = | <> | < | > | <= | >=
Exp		::= + Term TermSeq | - Term TermSeq | Term TermSeq
TermSeq		::= TermSeq + Term | TermSeq - Term | epsilon		//此处文法与原文法略有不同，但定义同一个语言。
Term		::= Factor FactorSeq
FactorSeq	::= FactorSeq * Factor | FactorSeq / Factor | epsilon	//此处文法与原文法略有不同，但定义同一个语言。
Factor		::= ident | number | ( Exp )
*/

//////////////////////////程序验证中使用的幽灵变量和逻辑定义////////////////////////
//@ ghost typedef long symArray[];
//@ ghost extern symArray symStream;

//@ ghost extern integer symStreamIndex;

/*@ //与文法对应的谓词定义
    //last == first-1对应文法中的空选择epsilon。对于文法中的epsilon，没有在symStream数组中增加任何符号。因为first和last分别是针对一个非终结符的第1个和最后1个符号的下标，
    //那么last == first-1就在情理之中了。而且这个last就是和它相邻的左边非终结符最后一个符号在symStream。
inductive Exp(symArray symStream, integer first, integer last);
inductive Factor(symArray symStream, integer first, integer last) =
    last == first && symStream[first] == ident ||
    last == first && symStream[first] == number || 
    last >= first+2 && symStream[first] == lparen && Exp(symStream, first+1, last-1) && symStream[last] == rparen;
inductive FactorSeq(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+1 && (\exists integer middle:[first..last-1]. FactorSeq(symStream, first, middle-1) &&
    		(symStream[middle] == times || symStream[middle] == slash) && Factor(symStream, middle+1, last));
inductive Term(symArray symStream, integer first, integer last) =
    last >= first && (\exists integer middle:[first..last]. Factor(symStream, first, middle) && FactorSeq(symStream, middle+1, last));
inductive TermSeq(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+1 && (\exists integer middle:[first..last-1].TermSeq(symStream, first, middle-1) &&
		(symStream[middle] == plus || symStream[middle] == minus) && Term(symStream, middle+1, last));
inductive Exp(symArray symStream, integer first, integer last) =
    last >= first+1 && (symStream[first] == plus || symStream[first] == minus) && 
	(\exists integer middle:[first+1..last]. Term(symStream, first+1, middle) && TermSeq(symStream, middle+1, last)) ||
    last >= first && symStream[first] != plus && symStream[first] != minus &&
	(\exists integer middle:[first..last]. Term(symStream, first, middle) && TermSeq(symStream, middle+1, last));
predicate RelOp(long sym) =
    sym == eql || sym == neq || sym == lss || sym == leq || sym == gtr || sym == geq;
predicate Cond(symArray symStream, integer first, integer last) =
    last >= first+1 && symStream[first] == oddsym && Exp(symStream, first+1, last) ||
    last >= first+2 && symStream[first] != oddsym && (\exists integer middle:[first+1..last-1].
	Exp(symStream, first, middle-1) && RelOp(symStream[middle]) && Exp(symStream, middle+1, last));
inductive StmtSeq(symArray symStream, integer first, integer last);
inductive Stmt(symArray symStream, integer first, integer last) =
    last == first-1 ||													// 空语句
    last >= first+2 &&symStream[first] == ident && symStream[first+1] == becomes && Exp(symStream, first+2, last) ||	// 赋值语句
    last == first+1 && symStream[first] == callsym && symStream[last] == ident ||					// 调用语句
    last >= first+2 && symStream[first] == ifsym && (\exists integer middle:[first+2..last].				// 条件语句
	Cond(symStream, first+1, middle-1) && symStream[middle] == thensym && Stmt(symStream, middle+1, last)) ||
    last >= first+2 && symStream[first] == whilesym && (\exists integer middle:[first+2..last].				// 循环语句
	Cond(symStream, first+1, middle-1) && symStream[middle] == dosym && Stmt(symStream, middle+1, last)) ||
    symStream[first] == beginsym && StmtSeq(symStream, first+1, last-1) && symStream[last] == endsym && last >= first+1;// 复合语句
inductive StmtSeq(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+1 && Stmt(symStream, first, last) ||
    last >= first && (\exists integer middle:[first..last].
	StmtSeq(symStream, first, middle-1) && symStream[middle] == semicolon && Stmt(symStream, middle+1, last));
predicate ConstDef(symArray symStream, integer first, integer last) =
    last == first+2 && symStream[first] == ident && symStream[first+1] == eql && symStream[first+2] == number;
inductive ConstDefSeq(symArray symStream, integer first, integer last) =
    last == first+2 && ConstDef(symStream, first, last) ||
    last >= first+6 && ConstDefSeq(symStream, first, last-4) && symStream[last-3] == comma && ConstDef(symStream, last-2, last);
predicate ConstDecl(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+4 && symStream[first] == constsym && ConstDefSeq(symStream, first+1, last-1) && symStream[last] == semicolon;
inductive VarDecSeq(symArray symStream, integer first, integer last) =
    last == first && symStream[first] == ident ||
    last >= first+2 && VarDecSeq(symStream, first, last-2) && symStream[last-1] == comma && VarDecSeq(symStream, last, last);
predicate VarDecl(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+2 && symStream[first] == varsym && VarDecSeq(symStream, first+1, last-1) && symStream[last] == semicolon;
inductive Block(symArray symStream, integer first, integer last);
inductive ProcDecl(symArray symStream, integer first, integer last) =
    last >= first+3 && symStream[first] == procsym && symStream[first+1] == ident && symStream[first+2] == semicolon &&
	Block(symStream, first+3, last-1) && symStream[last] == semicolon;
inductive ProcDecSeq(symArray symStream, integer first, integer last) =
    last == first-1 ||
    last >= first+3 && (\exists integer middle:[first..last]. ProcDecSeq(symStream, first, middle-1) && ProcDecl(symStream, middle, last));
inductive Block(symArray symStream, integer first, integer last) =
    last >= first-1 && (\exists integer middle1:[first-1..last]. \exists integer middle2:[first-1..last].
	\exists integer middle3:[first-1..last].
	    ConstDecl(symStream, first, middle1) && VarDecl(symStream, middle1+1, middle2) &&
		ProcDecSeq(symStream, middle2+1, middle3) && Stmt(symStream, middle3+1, last));
predicate Program(symArray symStream, integer first, integer last) =
    last >= first && Block(symStream, first, last-1) && symStream[last] == period;
*/

/*@ // 定义各种语法构造的中间代码结构的谓词，忽略了其中变量相对地址和常量值。
    // last == first-1也是对应文法中的空选择。和语法分析不同的是，对于代码生成而言，若是空选择，它根本不调用gen函数，不会改变Code数组中的内容，因而可以忽略这个选项。为避免忽略空语句选择引起多个函数的标注需要修改，我们暂不做修改。
inductive ExpCode(instruction code[cxmax+1], long first, long last);
inductive FactorCode(instruction code[cxmax+1], long first, long last) =
    last == first && code[first].f == lit && code[first].l == 0 ||
    last == first && code[first].f == lod ||
    last >= first && ExpCode(code, first, last);
inductive FactorSeqCode(instruction code[cxmax+1], long first, long last) =
    last == first-1 ||
    last >= first+1 && (\exists long middle:[first..last-1].	
	FactorSeqCode(code, first, middle-1) && FactorCode(code, middle, last-1) && code[last].f == opr &&
	    code[last].l == 0 && (code[last].a == 4 || code[last].a == 5));
inductive TermCode(instruction code[cxmax+1], long first, long last) =
    last >= first && (\exists long middle:[first..last].FactorCode(code, first, middle) && FactorSeqCode(code, middle+1, last));
inductive TermSeqCode(instruction code[cxmax+1], long first, long last) =
    last == first-1 ||
    last >= first+1 && (\exists long middle:[first..last-1].
	TermSeqCode(code, first, middle-1) && TermCode(code, middle, last-1) && code[last].f == opr &&
	    code[last].l == 0 && (code[last].a == 2 || code[last].a == 3));
inductive ExpCode(instruction code[cxmax+1], long first, long last) =
    last >= first+1 && (\exists long middle:[first+1..last].(TermCode(code, first, middle-1) &&
	    code[middle].f == opr && code[middle].l == 0 && code[middle].a == 1 && TermSeqCode(code, middle+1, last))) ||
    last >= first && (\exists long middle:[first..last].TermCode(code, first, middle) && TermSeqCode(code, middle+1, last));
predicate CondCode(instruction code[cxmax+1], long first, long last) = 
    last >= first+2 && (\exists long middle:[first..last-1].ExpCode(code, first, middle) && ExpCode(code, middle+1, last-1))&&
	code[last].f == opr && code[last].l == 0 &&
	    (code[last].a == 8 || code[last].a == 9 || code[last].a == 10 ||
                code[last].a == 11 || code[last].a == 12 || code[last].a == 13) ||
    last >= first+1 && ExpCode(code, first, last-1) && code[last].f == opr && code[last].l == 0 && code[last].a == 6;   
inductive StmtSeqCode(instruction code[cxmax+1], long first, long last);
inductive StmtCode(instruction code[cxmax+1], long first, long last) =
    last == first-1 ||														// 空语句
    last >= first+1 && ExpCode(code, first, last-1) && code[last].f == sto ||							// 赋值语句
    last == first && code[first].f == cal ||											// 过程调用语句
    last >= first+2 && (\exists long middle:[first..last-1].(CondCode(code, first, middle) &&				// 条件语句
	 code[middle+1].f == jpc && code[middle+1].l == 0 && code[middle+1].a == last+1 && StmtCode(code, middle+2, last))) ||
    last >= first+3 && (\exists long middle:[first..last-2].(CondCode(code, first, middle) &&				// while循环语句
	 code[middle+1].f == jpc && code[middle+1].l == 0 && code[middle+1].a == last+1 && StmtCode(code, middle+2, last-1)) &&
	    code[last].f == jmp && code[last].l == 0 && code[last].a == first) ||
    last >= first && StmtSeqCode(code, first, last);										// 语句序列
inductive StmtSeqCode(instruction code[cxmax+1], long first, long last) =
    last >= first-1  &&  StmtCode(code, first, last) ||
    last >= first && (\exists long middle:[first..last].StmtSeqCode(code, first, middle) && StmtCode(code, middle+1, last));
inductive BlockCode(instruction code[cxmax+1], long first, long last);
inductive ProcCode(instruction code[cxmax+1], long first, long last) = last >= first+1 && BlockCode(code, first, last);	
inductive ProcSeqCode(instruction code[cxmax+1], long first, long last) =
    last == first-1 ||
    last>=first+2 && (\exists long middle:[first-1..last-2].ProcSeqCode(code, first, middle) && ProcCode(code, middle+1, last));
inductive BlockCode(instruction code[cxmax+1], long first, long last) =
    last >= first+2 && code[first].f == jmp && code[first].l == 0 &&
    (\exists long middle:[first..last-1]. code[first].a == middle+1 && 
	ProcSeqCode(code, first+1, middle) &&
	code[middle+1].f == Int && code[middle+1].l == 0 &&
	StmtCode(code, middle+2, last-1)) &&
    code[last].f == opr && code[last].l == 0 && code[last].a == 0;
predicate ProgramCode(instruction code[cxmax+1], long first, long last) = last >= first+1 && BlockCode(code, first, last);
*/

// 下面这个逻辑函数用于函数getsym的验证。
/*@ logic long power(long m,long n) = 
    n == 0 ? 1 : m * power(m, n-1);
*/
/*@predicate currentSymbolFunc(long sym, char id[al+1], long num)=
    sym == beginsym || sym == endsym || sym == ifsym || sym == thensym ||
    sym == whilesym || sym == dosym || sym == callsym || sym == constsym ||
    sym == varsym || sym == procsym || sym == oddsym || 
    sym == ident && \is_pstring(id, al) || sym == number && 0 <= num < power(10, nmax) ||
    sym == neq || sym == lss || sym == leq || sym == gtr || sym == geq || sym==becomes ||
    sym == plus || sym == minus || sym == times || sym == slash || sym == lparen ||
    sym == rparen || sym == comma || sym == semicolon || sym == period || sym == eql ||
    sym == nul;
*/

/*@ define currentCodeLine: 0 < cc <= 80 && 0 < ll <= 80 && cc <= ll &&
	(\forall long i:[1..ll-1].0 <= line[i] <= 255) && ch == line[cc] && line[ll] == ' ' &&
	\length(infile) == 1 && \offset(infile) == 0 && (infile->_flag & _IOREAD) != 0;
    define currentCodeLineAndExtra: currentCodeLine && 0 <= cx <= cxmax+1 && \is_pstring(a, al) && (\forall long m:[kk..al-1]. a[m] == ' ');
    define currentSymbol:currentSymbolFunc(sym, id, num);
*/

/*@
predicate is_alpha(char ch) = 65 <= ch <= 90 || 97 <= ch <= 122;
predicate is_digit(char ch) = 48 <= ch <= 57;
*/

/*@
logic long a_to_i(char t[81], long first, long k) =
    (k == 0) ? 0 : a_to_i(t, first, k-1)*10 + t[first+k-1] - '0';
*/

/*@ 
ghost extern _Bool too_many_error, stop_here, nested_level_too_shallow;
*/


////////////////////////////////函数协议/////////////////////////////////////
/*@
requires 0 < cc <= ll && cc <= 80 && ll <= 80;
assigns err, too_many_error;
ensures 0 < cc <= ll;
exits \exit_status == 1 && too_many_error == true;
*/
void error(const long n);


//@ ghost  extern _Bool program_incomplete, program_line_overflow, too_many_variables;
/*@												// 暂时用这个多命名行为的函数协议。
requires 0 <= cc <= 80 && 0 <= ll <=80 && cc <= ll && \old(cc) == cc && \old(ll) == ll &&	//若仅有两个\old断言而无下一行的断言，则下面的完备性消失。可按此修改作为另一个测试用例。
	    \old(line[0..80]) == line[0..80] && \old(program_incomplete) == program_incomplete &&
	    \old(program_line_overflow) == program_line_overflow && 
	    \length(infile) == 1 && \offset(infile) == 0 && (infile->_flag & _IOREAD) != 0 && (cc == ll || cc > 0);
assigns cc, ch;	    
ensures currentCodeLine && ch == line[cc];
  behavior new_line:							// 当前行已结束，读入新的一行。
    assumes \old(cc) == \old(ll);
    requires \true;
    assigns ll, line[0..80], *infile, program_incomplete, program_line_overflow;
    ensures cc == 1;
    exits \exit_status == 1 && (cc == ll && program_incomplete == true || ll == 80 && program_line_overflow == true || ch == -1 );
  behavior current_line:						// 当前行还有剩余字符。
    assumes 0 < \old(cc) < \old(ll);
    requires currentCodeLine;
    ensures 1 < cc <= ll && cc == \old(cc)+1 && \old(line[0..80]) == line[0..80] && \old(ll) == ll &&
	    \old(program_incomplete) == program_incomplete && \old(program_line_overflow) == program_line_overflow;
  disjoint behaviors;
  complete behaviors;
*/
// 增加了检查输入数据每行不超过80个字符的代码和代码和断言，以保证line数组不会溢出。
void getch();

//@ ghost extern _Bool program_too_long;
// getsym本是一个可以写成多命名行为协议的函数，但是由于每次读完一个单词之后，输入流的指针指在该单词之后的下一个字符，它可能是空格或制表符，
// 导致getsym函数不能根据输入流的指针所指向的字符，来决定即将读入的单词属于单词分类中的哪个子集。
/*@ requires \old(symStreamIndex) == symStreamIndex && symStreamIndex >= -1 && currentCodeLine && ch == line[cc];
    assigns sym, a[0..al], id[0..al], kk, num, program_too_long, symStreamIndex, symStream[\old(symStreamIndex)+1], cc, ll, line[0..80], ch, err, *infile,  
	    program_incomplete, program_line_overflow, too_many_error;	//从cc开始，是间接被赋值的外部变量。
    ensures symStreamIndex >= 0 && symStreamIndex == \old(symStreamIndex) + 1 && symStream[symStreamIndex] == sym && currentCodeLine && currentSymbol;
    exits \exit_status == 1 && program_too_long == true;
*/
void getsym();

/*@
requires currentCodeLine && \old(cx) == cx && (\forall long i:[0..\old(cx)-1].\old(code[i]) == code[i]);
assigns cx, code[\old(cx)], program_too_long;		// cx有强类型不变式。
ensures currentCodeLine && cx == \old(cx)+1 && (\forall long i:[0..\old(cx)-1].\old(code[i]) == code[i]) &&
    code[\old(cx)].f == x && code[\old(cx)].l == y && code[\old(cx)].a == z;
exits \exit_status == 1 && program_too_long == true;
*/
void gen(const enum fct x, const long y, const long z);

/*@
requires currentCodeLineAndExtra && \old(s1) == s1 && \old(err) == err && \old(too_many_error) == too_many_error && \old(stop_here) == stop_here && \old(sym)== sym;
  behavior success:
    assumes (\old(sym) & \old(s1)) != 0;
    ensures  currentCodeLineAndExtra && \old(err) == err && \old(too_many_error) == too_many_error && \old(stop_here) == stop_here && \old(sym)== sym;
  behavior failure:
    assumes (\old(sym) & \old(s1)) == 0;
    assigns err, too_many_error, stop_here;	  
    ensures \true; 					//currentCodeLineAndExtra;
    exits   \exit_status == 2 && stop_here == true; 	//(\old(sym) & \old(s1)) == 0 &&
  complete behaviors;
  disjoint behaviors;
*/
void test(long s1, long const s2, long n); 

//@ ghost extern _Bool symbol_table_too_short;
/*@											// 用多命名行为的函数协议。
requires currentCodeLine && \is_pstring(id, al) && \old(tx) == tx &&			// table有强类型不变式，tx有强数据不变式。
	\old(lev) == lev && \old(dx) == dx && \old(too_many_variables) == too_many_variables &&
	\forall long i:[1..tx].\string(table[i].name, al) == \old(\string(table[i].name, al));
assigns tx, err, table[\old(tx)+1], symbol_table_too_short, too_many_error;
ensures currentCodeLine && table[tx].kind == k &&  
    (\forall long i:[1..tx-1].\string(table[i].name, al) == \old(\string(table[i].name, al))) && \string(table[tx].name, al) == \string(id, al);
exits \exit_status == 1 && symbol_table_too_short == true;
  behavior constant_id:
    assumes k == constant;
    ensures table[tx].val == num && \old(lev) == lev && \old(dx) == dx && tx == \old(tx)+1 && \old(too_many_variables) == too_many_variables;
  behavior variable_id:
    assumes k == variable;
    assigns dx, too_many_variables;
    ensures table[tx].level == lev && table[tx].addr == dx-1 && tx == \old(tx)+1;
    exits \exit_status == 1 && too_many_variables == true;
  behavior proc_id:
    assumes k == proc;
    requires \old(dx) == dx;
    ensures table[tx].level == lev && \old(dx) == dx && tx == \old(tx)+1 && \old(too_many_variables) == too_many_variables;
  complete behaviors;									// 正交性的证明依据k的类型。
  disjoint behaviors;
*/
void enter(const enum object k);

/*@
requires \is_pstring(id1, al) && \forall long i:[1..tx].\is_pstring(table[i].name, al) && \string(table[i].name, al) == \old(\string(table[i].name, al));
assigns table[0].name[0..al];
ensures 1 <= \result <= tx && \is_pstring(id1, al) && (\forall long i:[1..tx].\is_pstring(table[i].name, al) && \string(table[i].name, al) == \old(\string(table[i].name, al))) &&
	    \string(table[\result].name, al) == \string(id1, al) ||
	\result == 0 && \is_pstring(id1, al) && (\forall long i:[1..tx].\is_pstring(table[i].name, al) && \string(table[i].name, al) == \old(\string(table[i].name, al))) &&
	    (\forall long n:[1.. tx]. \string(table[n].name, al) != \string(id1, al));
*/
long position(const char* const id1);

/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(tx) == tx;
assigns	num, stop_here, a[0..al], id[0..al], kk, tx, table[\old(tx)+1], cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\old(symStreamIndex)+3], *infile,
	line[0..80], ch, program_incomplete, program_line_overflow, program_too_long, symbol_table_too_short, lev, too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && ConstDef(symStream, \old(symStreamIndex), symStreamIndex-1) && symStream[symStreamIndex] == sym &&
	    symStreamIndex >= 0 && symStreamIndex == \old(symStreamIndex)+3 && \string(table[tx].name, al) == \string(id, al) &&
	    table[tx].kind == constant && table[tx].val == num && 0 <= num < power(10, nmax) && tx == \old(tx)+1;
exits \exit_status == 2 && stop_here == true;
*/
void constdeclaration();

/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(tx) == tx;
assigns	stop_here, too_many_variables, a[0..al], id[0..al], kk, tx, table[\old(tx)+1], cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1], *infile, 
	num, line[0..80], ch, program_incomplete, program_line_overflow, too_many_variables, program_too_long, symbol_table_too_short, dx, lev, too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && VarDecSeq(symStream, \old(symStreamIndex), symStreamIndex - 1) &&
	    symStreamIndex >= 0 && symStreamIndex == \old(symStreamIndex) + 1 && symStream[symStreamIndex] == sym &&
	    \string(table[tx].name, al) == \string(id, al) && table[tx].kind == variable &&
	    table[tx].level == lev && table[tx].addr == dx-1 && tx > \old(tx);
exits \exit_status == 2 && stop_here == true;
*/
void vardeclaration();

/*@
requires 0 <= cx0 < cx;
ensures 0 <= cx0 < cx;
*/
void listcode(const long cx0);


/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(cx) == cx;
assigns	a[0..al], id[0..al], kk, cx, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax], table[0].name[0..al], *infile,
        num, line[0..80], ch, program_incomplete, program_line_overflow, program_too_long, stop_here, too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && Exp(symStream, \old(symStreamIndex), symStreamIndex-1) && symStreamIndex > \old(symStreamIndex) >= 0 &&
	    symStream[symStreamIndex] == sym && ExpCode(code, \old(cx), cx-1) && cx > \old(cx);
*/
void expression(const long fsys);

/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(cx) == cx;
assigns	num, stop_here, a[0..al], id[0..al], kk, cx, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax], *infile,
	line[0..80],ch,program_incomplete, program_line_overflow, program_too_long,table[0].name[0..al], too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && symStream[symStreamIndex] == sym &&
	    symStreamIndex > \old(symStreamIndex) >= 0 && Factor(symStream, \old(symStreamIndex), symStreamIndex-1) && FactorCode(code, \old(cx), cx-1) && cx > \old(cx);
exits \exit_status == 2 && stop_here == true || \exit_status == 1 && program_too_long == true;
*/
void factor(const long fsys);

/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(cx) == cx;
assigns	a[0..al], id[0..al], kk, cx, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax], num, line[0..80], ch, program_incomplete, *infile,
        program_line_overflow, program_too_long, stop_here, table[0].name[0..al], too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && symStream[symStreamIndex] == sym &&
    symStreamIndex > \old(symStreamIndex) >= 0 && Term(symStream, \old(symStreamIndex), symStreamIndex-1) && TermCode(code, \old(cx), cx-1) && cx > \old(cx);
*/
void term(const long fsys);


/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(cx) == cx;
assigns	stop_here, a[0..al], id[0..al], kk, cx, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax], *infile,
	num,line[0..80],ch,program_incomplete, program_line_overflow, table[0].name[0..al], program_too_long,too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && Cond(symStream, \old(symStreamIndex), symStreamIndex-1) && symStream[symStreamIndex] == sym && CondCode(code, \old(cx), cx-1) &&
	symStreamIndex > \old(symStreamIndex) >= 0 && cx > \old(cx);
exits \exit_status == 2 && stop_here == true;
*/
void condition(const long fsys);


/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym && \old(cx) == cx;
assigns stop_here, program_too_long, a[0..al], id[0..al], kk, cx, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax], *infile,
        num, line[0..80], ch, program_incomplete, program_line_overflow, table[0].name[0..al], too_many_error;
ensures currentCodeLineAndExtra && currentSymbol && Stmt(symStream, \old(symStreamIndex), symStreamIndex-1) && StmtCode(code, \old(cx), cx-1) && symStream[symStreamIndex] == sym &&
	symStreamIndex >= \old(symStreamIndex) >= 0 && cx >= \old(cx);
exits \exit_status == 2 && stop_here == true || \exit_status == 1 && program_too_long == true;
*/
void statement(const long fsys);


/*@
requires currentCodeLineAndExtra && currentSymbol && \old(symStreamIndex) == symStreamIndex && symStreamIndex >= 0 && symStream[symStreamIndex] == sym &&
	    \old(cx) == cx && \old(tx) == tx;
assigns stop_here, nested_level_too_shallow, dx, cx, tx, table[\old(tx)].addr, table[\old(tx)+1..txmax], lev, *infile,
	a[0..al], id[0..al], kk, cc, ll, err, sym, symStreamIndex, symStream[\old(symStreamIndex)+1..\infinity], code[\old(cx)..cxmax],
	num, line[0..80], ch, program_incomplete, program_line_overflow, too_many_variables, program_too_long, symbol_table_too_short, too_many_error, table[0].name[0..al];
ensures currentCodeLineAndExtra && currentSymbol && Block(symStream, \old(symStreamIndex), symStreamIndex-1) && BlockCode(code, \old(cx), cx-1) && symStream[symStreamIndex] == sym &&
	symStreamIndex >= \old(symStreamIndex) >= 0 && cx > \old(cx);
exits \exit_status == 1 && nested_level_too_shallow == true || \exit_status == 2 && stop_here == true;
*/
void block(long fsys);

